#!/bin/csh -f

# This script is an example of how to set up processing for the 
# the Direct Braodcast IMAPP MODIS cloud mask.

# Originally written by Liam Gumley liam.gumley@ssec.wisc.edu March, 2002.
#
# Updated November 2002 for operational processing of MODIS Terra and
#  Aqua data - Kathy Strabala

# Search for !!! for places where directoy locations are set and will
#  need to be changed for on-site automated processing.

#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

# Check arguments
if ($#argv != 5) then
  echo "Usage: modis_cloudmask.csh SAT FIL1KM FILQKM FILGEO OUTDIR"
  echo "where"
  echo "SAT is the MODIS platform name (Terra or Aqua)"
  echo "FIL1KM is MODIS L1B 1000 meter resolution HDF image file"
  echo "FILQKM is MODIS L1B 250  meter resolution HDF image file"
  echo "FILGEO is MODIS L1B 1000 meter resolution HDF geolocation file"
  echo "OUTDIR is the directory where products will be stored"
  exit(-1)
endif

# Extract file names
set SAT = $argv[1]
set FIL1KM = $argv[2]
set FILQKM = $argv[3]
set FILGEO = $argv[4]
set OUTDIR = $argv[5]

# Set root of output file name (e.g. 't1.02001.1815')
set ROOT = $FIL1KM:r
set ROOT = $ROOT:r
set ROOT = $ROOT:t

#Convert Satellite name to standard that threshold file will recognize
set fl=`echo $SAT|cut -c1`
if ( "$fl" == "T"  || "$fl" == "t") then
   set SAT = "Terra"
else if ( "$fl" == "A"  || "$fl" == "a" ) then
     set SAT = "Aqua"
else
     echo "Satellite name incorrect" $SAT
     exit 1
endif

#-------------------------------------------------------------------------------
# GET ANCILLARY DATA IF NECESSARY
# 
#  If you are not processing a data set where snow and ice
#  are present, you can delete or comment out the the 
#  NISE and NCEP_ICE portions of this section.

#  Works off of ancillary data sets in the given directory.

#  All dynamic ancillary data sets are available real-time at: 
#   ftp://terra.ssec.wisc.edu/pub/terra/ancillary
#-------------------------------------------------------------------------------

#!!!!!!! Set directory for ancillary data and source !!!!!!!!!!
set ANC_DIR = ../../data
set ANC_SRC = ../../ancillary/src

# Set default names for unpacked NISE files
set NISE_NORTH_BIN = ""
set NISE_SOUTH_BIN = ""

set NISE_FILE = `find $ANC_DIR -name "NISE_SSMIF13_????????.HDFEOS" | sort | tail -1`
if ($NISE_FILE == "") then
  echo "No NISE files were found in "$ANC_DIR
  exit(1)
endif

# Point to and unpack the NISE data file
set NISE_NORTH_BIN = ${NISE_FILE:t}_NORTH.bin
set NISE_SOUTH_BIN = ${NISE_FILE:t}_SOUTH.bin
$ANC_SRC/extract_nsidc_nise.csh $NISE_FILE $NISE_NORTH_BIN $NISE_SOUTH_BIN
endif
  
# Set default name for unpacked NCEP ice file
set NCEP_ICE_BIN = ""

# Get name of correct ncep ICE file (should be only one in directory) 
set NCEP_ICE_FILE = `find $ANC_DIR -name "eng.??????" | sort | tail -1`
if ($NCEP_ICE_FILE == "") then
  echo "No NCEP ice files were found in "$ANC_DIR
  exit(1)
endif

set NCEP_ICE_BIN = ${NCEP_ICE_FILE:t}.bin
$ANC_SRC/extract_ncep_ice.csh $NCEP_ICE_FILE $NCEP_ICE_BIN

#------------------------------------------------------------------------------
# RUN Band 26 DESTRIPING ALGORITHM - Currently TERRA only
#------------------------------------------------------------------------------

#Destripe band 26 data - if terra
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Started MODIS 1KM destriping at "`date`
echo "MODIS L1B 1KM: "$FIL1KM

#!!!!!! Set directory for destripe data and src !!!!!!!!!
set SRCDIR = ../../destripe_band26/src
set DATA_DIR = ../../data
set COEFF_DIR = ../../destripe_band26/coeff
$SRCDIR/correct_band26.exe $SAT $DATA_DIR/$FIL1KM $COEFF_DIR/terra.b26_coefficients.2002_07_02

#-------------------------------------------------------------------------------
# RUN CLOUD MASK ALGORITHM
#-------------------------------------------------------------------------------

# Print start message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Started MODIS cloud mask processing at "`date`
echo "MODIS L1B 1KM: "$FIL1KM
echo "MODIS L1B QKM: "$FILQKM
echo "MODIS L1B GEO: "$FILGEO

# Extract flat files
echo "Started flatfile extraction at "`date`

#!!!!!! Set directory for flatfile extraction data and src !!!!!!!!!
set SRCDIR = ../../flatfile/src
set DATA_DIR = ../../data

$SRCDIR/modis_extract_1km.exe $DATA_DIR/$FIL1KM $SAT ${ROOT}.1000m.img ${ROOT}.1000m.hdr
$SRCDIR/modis_extract_qkm.exe $DATA_DIR/$FILQKM $SAT ${ROOT}.250m.img ${ROOT}.250m.hdr
$SRCDIR/modis_extract_geo.exe $DATA_DIR/$FILGEO $SAT ${ROOT}.geo.img ${ROOT}.geo.hdr
$SRCDIR/modis_extract_met.exe $DATA_DIR/$FIL1KM $SAT ${ROOT}.met.img ${ROOT}.met.hdr
echo "Finished flatfile extraction at "`date`

# Set output file names
set OUTFIL      = $ROOT.mod35.img
set OUTFILQA    = $ROOT.mod35qa.img
set OUTFILHDR   = $ROOT.mod35.hdr
set OUTFILQAHDR = $ROOT.mod35qa.hdr
set DEBUG       = $ROOT.mod35debug.txt

# Create a configuration file from the template
# !!!!! Note: location of threshold file needs to be set below !!!!
set TEMPLATE = ../cmd/cloudmask.cfg
/bin/sed \
  -e "s/FIL1KM_IMG/${ROOT}.1000m.img/g" \
  -e "s/FIL1KM_HDR/${ROOT}.1000m.hdr/g" \
  -e "s/FILQKM_IMG/${ROOT}.250m.img/g" \
  -e "s/FILQKM_HDR/${ROOT}.250m.hdr/g" \
  -e "s/FILGEO_IMG/${ROOT}.geo.img/g" \
  -e "s/FILGEO_HDR/${ROOT}.geo.hdr/g" \
  -e "s/FILMET_IMG/${ROOT}.met.img/g" \
  -e "s/FILMET_HDR/${ROOT}.met.hdr/g" \
  -e "s/NISE_NORTH/${NISE_NORTH_BIN}/g" \
  -e "s/NISE_SOUTH/${NISE_SOUTH_BIN}/g" \
  -e "s/NCEP_ICE/${NCEP_ICE_BIN}/g" \
  -e "s/THRESHOLD_FILE/..\/cmd\/thresholds.dat.${SAT}/g" \
  -e "s/MOD35.img/${OUTFIL}/g" \
  -e "s/MOD35QA.img/${OUTFILQA}/g" \
  -e "s/MOD35.hdr/${OUTFILHDR}/g" \
  -e "s/MOD35QA.hdr/${OUTFILQAHDR}/g" \
  -e "s/debug.dat/${DEBUG}/g" \
  $TEMPLATE > cloudmask.cfg

# Run the cloud mask
set SRCDIR = ../cmd
$SRCDIR/cloudmask.exe cloudmask.cfg $SAT

# Rename the configuration file
/bin/mv cloudmask.cfg $ROOT.mod35.cfg

# Move the output files to the product directory
/bin/mv $OUTFIL $OUTFILQA $OUTFILHDR $OUTFILQAHDR $DEBUG $ROOT.mod35.cfg $OUTDIR

#-------------------------------------------------------------------------------
# CLEANUP AND EXIT
#-------------------------------------------------------------------------------

# Print end message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Finished MODIS cloud mask product processing at "`date`

# Exit gracefully
exit(0)
