C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      integer function  get_angles(
     &                              cube,
     &                              scan_type,
     &                              max_sol,
     &                              min_sol,
     &                              lun_geo,
     &                              datatype_geo,
     &                              interleave_geo,
     &                              resolution_geo,
     &                              error_geo,
     &                              offset_geo,
     &                              samples_geo,
     &                              lines_geo,
     &                              bands_geo,
     &                              bandnames_geo,
     &                              bandunits_geo,
     &                              rlat,
     &                              rlon,
     &                              elev,
     &                              view_zen,
     &                              solar_zen,
     &                              cube_topog,
     &                              rel_angle
     &                             )
      implicit none
      save

c----------------------------------------------------------------------
c!F77
c
c!Description:
c     Routine for reading and putting MODIS cloud mask geometry
c     values into arrays for processing.
c
c!Input Parameters:
c cube          Scan cube counter
c scan_type     Day or night mode of scan
c max_sol       Maximum solar zenith angle of this granule
c min_sol       Minimum solar zenith anlge of this granule
c buf_geo_size  Line and element granule size
c
c!Output Parameters:
c
c rlat          Array containing scan cube of 1km pixel latitude
c               values
c rlon          Array containing scan cube of 1km pixel longitude
c               values
c elev          Array containing scan cube of 1km pixel elevation
c               values
c view_zen      Array containing scan cube of 1km pixel viewing
c               zenith angles
c solar_zen     Array containing scan cube of 1km pixel solar
c               azimulth angles
c cube_topog    Array containing scan cube of 1 km land/sea
c               information
c rel_angle     Array containing scan cube of 1km pixel relative
c               angles
c
c!Revision History:
c
c!Team-Unique Header:
c
c!References and Credits:
c See Cloud Mask ATBD-MOD-06.
c Revision 01.00  1996/2/13 08:20:37
c K. Strabala (kathys@ssec.wisc.edu)
c Original version.
c
c!END
c----------------------------------------------------------------------

      include 'global.inc'

c --- constants
      real 		PI
      parameter		(PI = 3.14159)

      real 		DTR
      parameter		(DTR = PI/180.0)

      real 		RTD
      parameter		(RTD = 180.0/PI)

c --- parameters
      character*4 	interleave_geo
      character*80	bandnames_geo(20)
      character*80      bandunits_geo(20)
      character*1 	scan_type
      integer 		cube
      integer		lun_geo
      integer		datatype_geo
      integer		resolution_geo
      integer		offset_geo
      integer		samples_geo
      integer		lines_geo
      integer		bands_geo
      integer		cube_topog(NPIXEL,SCANS_CUBE)
      real 		max_sol
      real		min_sol
      real		error_geo
      real              rlat(NPIXEL,SCANS_CUBE)
      real		rlon(NPIXEL,SCANS_CUBE) 
      real		view_zen(NPIXEL,SCANS_CUBE)
      real		solar_zen(NPIXEL,SCANS_CUBE) 
      real              rel_angle(NPIXEL,SCANS_CUBE)
      real              elev(NPIXEL,SCANS_CUBE)

c --- external functions
      integer 		read_flat_file
      intrinsic         abs
      intrinsic         sin
      intrinsic         cos
      intrinsic         acos
      intrinsic         nint

c --- internal variables
      character*80	req_bandname
      character*80	req_bandunit
      byte 		flag(NPIXEL*SCANS_CUBE)
      integer		req_resolution
      integer 		status
      integer 		ElementSize
      integer 		LineSize
      integer 		i
      integer 		j
      integer 		k
      integer 		data_size(2)
      real              vzar
      real              szar
      real              razr
      real              rela
      real              cossna
      real 		buf(NPIXEL*SCANS_CUBE)
      real              sens_azim(NPIXEL,SCANS_CUBE)
      real              solar_azim(NPIXEL,SCANS_CUBE)
      real              rel_azim(NPIXEL,SCANS_CUBE)

c --- Common block used for writing debug output statements
      integer 		debug
      integer 		h_output
      common / bug / debug, h_output


c --- set function status to FAILED
      get_angles = -1

c --- debug
      if (debug .gt. 3) then
         write(h_output,'(10x/,''Within get_angles routine '',/)')
         write(h_output,'(10x/,''Scan mode is: '',A1,/)') scan_type
      endif


c --- initialize output paramaters
      data_size(1) = 0
      data_size(2) = 0
      ElementSize = samples_geo
      LineSize = lines_geo

      do i = 1 , NPIXEL
      do j = 1 , SCANS_CUBE
         rlat(i,j)       = BAD_DATA
         rlon(i,j)       = BAD_DATA
         elev(i,j)       = BAD_DATA
         view_zen(i,j)   = BAD_DATA
         solar_zen(i,j)  = BAD_DATA
         sens_azim(i,j)  = BAD_DATA
         solar_azim(i,j) = BAD_DATA
         rel_azim(i,j)   = BAD_DATA
         cube_topog(i,j) = -1
         rel_angle(i,j)  = 999.0
      enddo
      enddo


c ------------------------------------------------------------------
c --- Read the latitude values
      req_resolution = 1
      req_bandname   = 'Latitude'
      req_bandunit   = 'deg'

      do i = 1, NPIXEL*SCANS_CUBE
      buf(i)  = BAD_DATA
      enddo

      status = read_flat_file(
     &  	               cube,	 	
     &			       req_resolution,
     &			       req_bandname,
     &			       req_bandunit,
     &			       lun_geo,
     &			       datatype_geo,
     &			       interleave_geo,
     &			       resolution_geo,
     &                         error_geo,
     &			       offset_geo,
     &			       samples_geo,
     &			       lines_geo,
     &			       bands_geo,
     &			       bandnames_geo,
     &			       bandunits_geo,
     &			       ElementSize,
     &			       LineSize,
     &			       buf,
     &			       flag,
     &			       data_size(1),
     &			       data_size(2)
     &                       )

c --- check for error
      if( status.ne.0 )then
         call message ('get_angles',
     &   'Failed to extract latitude from Geo. file',
     &   0, 2 )
         return
      endif

c --- Place the data into the holding arrays
      k = 1
      do i = 1 , data_size(2)
      do j = 1 , data_size(1)
      if( buf(k).ge.-90.0 .and.buf(k).le.90.0 ) then
         rlat(j,i) = buf(k)
      endif
      k = k+1
      enddo
      enddo


c ------------------------------------------------------------------


c ------------------------------------------------------------------
c --- Read the longitude values
      req_resolution = 1
      req_bandname   = 'Longitude'
      req_bandunit   = 'deg'

      do i = 1, NPIXEL*SCANS_CUBE
      buf(i)  = BAD_DATA
      enddo

      status = read_flat_file(
     &  	               cube,	 	
     &			       req_resolution,
     &			       req_bandname,
     &			       req_bandunit,
     &			       lun_geo,
     &			       datatype_geo,
     &			       interleave_geo,
     &			       resolution_geo,
     &                         error_geo,
     &			       offset_geo,
     &			       samples_geo,
     &			       lines_geo,
     &			       bands_geo,
     &			       bandnames_geo,
     &			       bandunits_geo,
     &			       ElementSize,
     &			       LineSize,
     &			       buf,
     &			       flag,
     &			       data_size(1),
     &			       data_size(2)
     &                       )

c --- check for error
      if( status.ne.0 )then
         call message ('get_angles',
     &   'Failed to extract longitude from Geo. file',
     &   0, 2 )
         return
      endif

c --- Place the data into the holding arrays
      k = 1
      do i = 1 , data_size(2)
      do j = 1 , data_size(1)
      if( buf(k).ge.-180.0 .and.buf(k).le.180.0 ) then
         rlon(j,i) = buf(k)
      endif
      k = k+1
      enddo
      enddo

c ------------------------------------------------------------------


c ------------------------------------------------------------------
c --- Read elevation values
      req_resolution = 1
      req_bandname   = 'Elevation'
      req_bandunit   = 'm  '

      do i = 1, NPIXEL*SCANS_CUBE
      buf(i)  = BAD_DATA
      enddo

      status = read_flat_file(
     &  	               cube,	 	
     &			       req_resolution,
     &			       req_bandname,
     &			       req_bandunit,
     &			       lun_geo,
     &			       datatype_geo,
     &			       interleave_geo,
     &			       resolution_geo,
     &                         error_geo,
     &			       offset_geo,
     &			       samples_geo,
     &			       lines_geo,
     &			       bands_geo,
     &			       bandnames_geo,
     &			       bandunits_geo,
     &			       ElementSize,
     &			       LineSize,
     &			       buf,
     &			       flag,
     &			       data_size(1),
     &			       data_size(2)
     &                       )

c --- check for error
      if( status.ne.0 )then
         call message ('get_angles',
     &   'Failed to extract elevation from Geo. file',
     &   0, 2 )
         return
      endif

c --- Place the data into the holding arrays
      k = 1
      do i = 1 , data_size(2)
      do j = 1 , data_size(1)
      if( buf(k).ge.-500.0 .and.buf(k).le.9000.0 ) then
         elev(j,i) = buf(k)
      endif
      k = k+1
      enddo
      enddo

c ------------------------------------------------------------------


c ------------------------------------------------------------------
c --- Read the solar zenith angle information
      req_resolution = 1
      req_bandname   = 'SolarZenith'
      req_bandunit   = 'deg'

      do i = 1, NPIXEL*SCANS_CUBE
      buf(i)  = BAD_DATA
      enddo

      status = read_flat_file(
     &  	               cube,	 	
     &			       req_resolution,
     &			       req_bandname,
     &			       req_bandunit,
     &			       lun_geo,
     &			       datatype_geo,
     &			       interleave_geo,
     &			       resolution_geo,
     &                         error_geo,
     &			       offset_geo,
     &			       samples_geo,
     &			       lines_geo,
     &			       bands_geo,
     &			       bandnames_geo,
     &			       bandunits_geo,
     &			       ElementSize,
     &			       LineSize,
     &			       buf,
     &			       flag,
     &			       data_size(1),
     &			       data_size(2)
     &                       )

c --- check for error
      if( status.ne.0 )then
         call message ('get_angles',
     &   'Failed to extract solar zenith from Geo. file',
     &   0, 2 )
         return
      endif

c --- Place the data into the holding arrays
      k = 1
      do i = 1 , data_size(2)
      do j = 1 , data_size(1)
      if( buf(k).ge.0.0 .and.buf(k).le.180.0 ) then
         solar_zen(j,i) = buf(k)

c ------ Determine max and min solar zenith angles of granule
         if( nint(solar_zen(j,i)).ne.-32767 .and.
     &       solar_zen(j,i).gt.max_sol)  max_sol = solar_zen(j,i)
         if (nint(solar_zen(j,i)) .ne. -32767 .and.
     &       solar_zen(j,i).lt.min_sol)  min_sol = solar_zen(j,i)

      endif
      k = k+1
      enddo
      enddo

c ------------------------------------------------------------------


c ------------------------------------------------------------------
c --- Read the sensor zenith angle information
      req_resolution = 1
      req_bandname   = 'SensorZenith'
      req_bandunit   = 'deg'

      do i = 1, NPIXEL*SCANS_CUBE
      buf(i)  = BAD_DATA
      enddo

      status = read_flat_file(
     &  	               cube,	 	
     &			       req_resolution,
     &			       req_bandname,
     &			       req_bandunit,
     &			       lun_geo,
     &			       datatype_geo,
     &			       interleave_geo,
     &			       resolution_geo,
     &                         error_geo,
     &			       offset_geo,
     &			       samples_geo,
     &			       lines_geo,
     &			       bands_geo,
     &			       bandnames_geo,
     &			       bandunits_geo,
     &			       ElementSize,
     &			       LineSize,
     &			       buf,
     &			       flag,
     &			       data_size(1),
     &			       data_size(2)
     &                       )

c --- check for error
      if( status.ne.0 )then
         call message ('get_angles',
     &   'Failed to extract sensor zenith from Geo. file',
     &   0, 2 )
         return
      endif

c --- Place the data into the holding arrays
      k = 1 
      do i = 1 , data_size(2)
      do j = 1 , data_size(1)
      if( buf(k).ge.0.0 .and.buf(k).le.180.0 ) then
         view_zen(j,i) = buf(k)
      endif
      k = k+1
      enddo
      enddo

c ------------------------------------------------------------------

c ------------------------------------------------------------------
c --- Read the Land Sea mask
      req_resolution = 1
      req_bandname   = 'LandSea'
      req_bandunit   = 'flg'

      do i = 1, NPIXEL*SCANS_CUBE
      buf(i)  = BAD_DATA
      enddo

      status = read_flat_file(
     &  	               cube,	 	
     &			       req_resolution,
     &			       req_bandname,
     &			       req_bandunit,
     &			       lun_geo,
     &			       datatype_geo,
     &			       interleave_geo,
     &			       resolution_geo,
     &                         error_geo,
     &			       offset_geo,
     &			       samples_geo,
     &			       lines_geo,
     &			       bands_geo,
     &			       bandnames_geo,
     &			       bandunits_geo,
     &			       ElementSize,
     &			       LineSize,
     &			       buf,
     &			       flag,
     &			       data_size(1),
     &			       data_size(2)
     &                       )

c --- check for error
      if( status.ne.0 )then
         call message ('get_angles',
     &   'Failed to extract land/sea mask from Geo. file',
     &   0, 2 )
         return
      endif

c --- Place the data into the holding arrays
      k = 1
      do i = 1 , data_size(2)
      do j = 1 , data_size(1)
      if( buf(k).ge.-90.0 .and.buf(k).le.90.0 ) then
         cube_topog(j,i) = nint( buf(k) )
      endif
      k = k+1
      enddo
      enddo

c ------------------------------------------------------------------


c --- Added check for day or night scan mode.  Read the
c     azimuth data and determine a relative anlge only
c     if in day mode
      if( scan_type(1:1).eq.'D' ) then

c ------------------------------------------------------------------
c ------ Read the sensor azimuth angle information
         req_resolution = 1
         req_bandname   = 'SensorAzimuth'
         req_bandunit   = 'deg'

         do i = 1, NPIXEL*SCANS_CUBE
         buf(i)  = BAD_DATA
         enddo

         status = read_flat_file(
     &  	                  cube,	 	
     &		 	          req_resolution,
     &			          req_bandname,
     &			          req_bandunit,
     &			          lun_geo,
     &			          datatype_geo,
     &			          interleave_geo,
     &			          resolution_geo,
     &                            error_geo,
     &			          offset_geo,
     &			          samples_geo,
     &			          lines_geo,
     &			          bands_geo,
     &			          bandnames_geo,
     &			          bandunits_geo,
     &			          ElementSize,
     &			          LineSize,
     &			          buf,
     &			          flag,
     &			          data_size(1),
     &			          data_size(2)
     &                          )

c ------ check for error
         if( status.ne.0 )then
            call message ('get_angles',
     &      'Failed to extract sensor azimuth from Geo. file',
     &      0, 2 )
         endif

c ------ Place the data into the holding arrays
         k = 1
         do i = 1 , data_size(2)
         do j = 1 , data_size(1)
         if( buf(k).ge.-180.0 .and.buf(k).le.180.0 ) then
            sens_azim(j,i) = buf(k)
         endif
         k = k+1
         enddo
         enddo

c ------------------------------------------------------------------


c ------------------------------------------------------------------
c ------ Read the solar azimuth angle information
         req_resolution = 1
         req_bandname   = 'SolarAzimuth'
         req_bandunit   = 'deg'

         do i = 1, NPIXEL*SCANS_CUBE
         buf(k)  = BAD_DATA
         enddo

         status = read_flat_file(
     &  	                  cube,	 	
     &		 	          req_resolution,
     &			          req_bandname,
     &			          req_bandunit,
     &			          lun_geo,
     &			          datatype_geo,
     &			          interleave_geo,
     &			          resolution_geo,
     &                            error_geo,
     &			          offset_geo,
     &			          samples_geo,
     &			          lines_geo,
     &			          bands_geo,
     &			          bandnames_geo,
     &			          bandunits_geo,
     &			          ElementSize,
     &			          LineSize,
     &			          buf,
     &			          flag,
     &			          data_size(1),
     &			          data_size(2)
     &                          )

c ------ check for error
         if( status.ne.0 )then
            call message ('get_angles',
     &      'Failed to extract solar azimuth from Geo. file',
     &      0, 2 )
            return
         endif

c ------ Place the data into the holding arrays
         k = 1
         do i = 1 , data_size(2)
         do j = 1 , data_size(1)
         if( buf(k).ge.-180.0 .and.buf(k).le.180.0 ) then
            solar_azim(j,i) = buf(k)
         endif
         k = k+1
         enddo
         enddo

c ------------------------------------------------------------------

c ------------------------------------------------------------------
c ------ compute the relative azimuth angle
         do i = 1 , data_size(2)
         do j = 1 , data_size(1)
            rel_azim(j,i) = abs(180.0-abs(sens_azim(j,i)-
     &                        solar_azim(j,i)))
         enddo
         enddo

c ------ compute the relative angle (value that sun glint is based upon.)
         do i = 1 , data_size(2)
         do j = 1 , data_size(1)
            if( nint(view_zen(j,i))   .ne. nint(bad_data) .and.
     &          nint(solar_azim(j,i)) .ne. nint(bad_data) .and.
     &          nint(solar_zen(j,i))  .ne. nint(bad_data) .and.
     &          nint(sens_azim(j,i))  .ne. nint(bad_data)
     &      ) then
                vzar = view_zen(j,i) * dtr
                szar = solar_zen(j,i) * dtr
                razr = rel_azim(j,i) * dtr
                cossna = sin(vzar) * sin(szar) * cos(razr) +
     +                   cos(vzar) * cos(szar)
                cossna = min(max(cossna,-1.0),1.0)
                rela = acos(cossna) * rtd
                rel_angle(j,i) = rela
            endif
         enddo
         enddo
c ------------------------------------------------------------------


      endif

c ... debug statement ............................................
      if( debug.gt.3) then
        write(h_output,'(15x,'' ANGLE VALUES: '',2i10)') data_size(1),
     *        data_size(2)
        write(h_output,'(2x,'' Latitude'',/,20(5f10.4/))')
     +       ((rlat(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Longitude'',/,20(5f10.4/))')
     +       ((rlon(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Elevation'',/,20(5f10.4/))')
     +       ((elev(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Solar Zenith'',/,20(5f10.4/))')
     +       ((solar_zen(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Viewing Zenith'',/,20(5f10.4/))')
     +       ((view_zen(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Solar Azimuth '',/,20(5f10.4/))')
     +       ((solar_azim(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Sensor Azimuth '',/,20(5f10.4/))')
     +       ((sens_azim(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Relative Angle'',/,20(5f10.4/))')
     +       ((rel_angle(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Land/Sea Mask'',/,20(5I10/))')
     +       ((cube_topog(j,i),j=1,10),i=1,10)
        write(h_output,'(15x,''MAX/MIN SOLAR ZENITH ANGLES:'',
     +        2f10.4,/)') max_sol,min_sol
      endif
c ................................................................

c --- set function status to SUCCESS
      get_angles = 0

      return
      end
