C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine PolarNite_land(pxldat,vza,testbits,qa_bits,
     +                          nmtests,confdnc)

      implicit none
      save

c---------------------------------------------------------------------
c!F77 
c
c!Description:
c      Routine for performing clear sky tests over polar land
c      surfaces during nightime hours.
c
c      For each threshold, a "low confidence" and a "high confidence"
c      cutoff value is defined ('locut','hicut').  Confidence is
c      0 % beyond 'locut' and 100% beyond 'hicut'.  Please note
c      that 'locut' may be arithmetically greater that 'hicut'.
c      For tests which use a range of values (like the visible ratio
c      test), two low cutoffs and two high cutoffs are required, one
c      pair for each end of the range.  In this case, the two pairs
c      must be entered in numeric order (lower pair in index 1 of
c      'locut' and 'hicut').
c
c      For nighttime polar land the groups are:
c          Group 1: High thick cloud
c                   6.75 micron bt test 
c
c          Group 2: Low cloud - thick
c                   11-12 micron bt tests
c                   11-4 micron bt tests
c
c          Group 5: High cloud - thin
c                   3.7-12 micron bt test
c
c!Input Parameters:
c pxldat        Array containing reflectance or brightness temperatures
c               for all bands for a single pixel
c vza           viewing zenith angle
c
c!Output Parameters:
c testbits      four-byte integer containing bit results
c qa_bits       10 byte array containing QA bit results
c nmtests       Number of tests actually applied in this routine
c confdnc       product of all applied individual confidences
c
c!Revision History:
c
c!Team-Unique Header:
c
c!References and Credits:
c See Cloud Mask ATBD-MOD-06.
c
c!Design Notes:
c
c Externals:
c       Subroutines: conf_test,set_bit,tview
c
c!END
c-----------------------------------------------------------------------

      include 'global.inc'
      include 'PolarNite_land_thr.inc'
c ...
c ... scalar arguments ..
      real confdnc,vza
      integer nmtests
c ...
c ... array arguments ..
      real pxldat(inband)
      byte testbits(6),qa_bits(10)
c ...
c ... local scalars ..
      real c2,mas4_12,masir11,masir12,masir13,masir4,masir8,
     +     c3,masir3,masir65,mas11_4,c4,cmin1,cmin2,cmin5,groups,
     +     fac,pre_confdnc,c5,masdf1,schi,cosvza,dtr,pi,diftemp,
     +     dfthrsh,locut,hicut,Rel_equality_EPS,midpt,power
      integer nptests,debug,h_output,kk

c ... local arrays
      integer ngtests(3)
c ...
      parameter(Rel_equality_EPS = 0.000001)

c ... external subroutines ..
      external conf_test,set_bit,tview

c ... Common statement for debug purposes
      common / bug / debug, h_output

c ...
c ... initialize variables
      pi = acos(-1.0)
      dtr = pi/180.0
c ... nmtests counts the number of tests applied to this pixel
      nmtests = 0
c ... nptests counts the number of tests passed
      nptests = 0
c ... confidence to 1.0 to begin with
      confdnc = 1.0

c ... place band values into individual variables for easy
c ... identification
      masir3 = pxldat(20)
      masir4 = pxldat(22)
      masir65 = pxldat(27)
      masir8 = pxldat(29)
      masir11 = pxldat(31)
      masir12 = pxldat(32)
      masir13 = pxldat(35)

c ...
      mas4_12 = 0.0
      mas11_4 = 0.0
      masdf1 = 0.0
      schi = 0.0

c ... the c suffix variables represent individual test confidences
      c2 = 0.0
      c3 = 0.0
      c4 = 0.0
      c5 = 0.0
      cmin1 = 1.0
      cmin2 = 1.0
      cmin5 = 1.0

c ... initialize group number holder
      do 10 kk = 1 , 3
         ngtests(kk) = 0
  10  continue

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(10x/,''Processing subroutine LandNite '',/)')
      endif
c ................................................................


C     **** GROUP 1 TESTS *************************************

c     H20 vapor channel (6.7 micron) high cloud test
      if (nint(masir65) .ne. nint(bad_data)) then
        nmtests = nmtests + 1
        call set_qa_bit(qa_bits,15)
        if (masir65 .gt. pnlh20(2)) then
          call set_bit(testbits,15)
          nptests = nptests + 1
        end if
        call conf_test(masir65,pnlh20(1),pnlh20(3),pnlh20(4),
     *                pnlh20(2),1,c2)
        cmin1 = min(cmin1,c2)
        ngtests(1) = ngtests(1) + 1
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(1x,''masir65: '',5f10.2)') masir65,pnlh20(1),
     +          pnlh20(2),pnlh20(3),pnlh20(4)
      endif
c ................................................................
c     *****  END OF GROUP 1 TESTS  ***************************
c
c 
c     ****  GROUP 2 TESTS  ***********************************
c ... 11-12um brightness temperature difference test
c ... for thin cirrus).
      if (nint(masir11) .ne. nint(bad_data) 
     +                 .and.
     +    nint(masir12) .ne. nint(bad_data)
     +                  .and.
     +              vza .gt. 0.0) then
      
        masdf1 = masir11 - masir12
c ...   added apollo viewing angle/av4t regressed threshold.
c ...   calculate secant of viewing zenith angle.
        cosvza = cos(vza*dtr)
        if (abs(cosvza).gt.Rel_equality_EPS) then
          schi = 1.0/cosvza
        else
          schi = 99.0
        end if

c ...   interpolate look-up table values of 11 - 12 micron bt
c ...   difference thresholds (function of viewing zenith
c ...   and 11 micron brightness temperature).
        call tview(1,schi,masir11,diftemp)

c ...   if a threshold was determined by apollo, then use this
c ...   as the thin cirrus test, otherwise use a standard threshold
        if (diftemp.lt.0.1 .or. abs(schi-99.0).lt.0.0001) then
          dfthrsh = pnl11_12hi(1)
        else
          dfthrsh = diftemp
        end if

c...    Set flags if test passed
        nmtests = nmtests + 1
        call set_qa_bit(qa_bits,18)
        if (masdf1.le.dfthrsh) then
          call set_bit(testbits,18)
          nptests = nptests + 1
        end if
        locut = dfthrsh + 0.5
        hicut = dfthrsh - 1.25
        call conf_test(masdf1,locut,hicut,1.0,dfthrsh,1,c5)
        cmin2 = min(cmin2,c5)
        ngtests(2) = ngtests(2) + 1
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(1x,''APOLLO masdf1: '',5f10.2)') masdf1,
     +          pnl11_12hi(1),dfthrsh,locut,hicut
      endif
c ................................................................

c ... 11 minus 4 micron BTDIF fog and low cloud test.
      if (nint(masir11) .ne. nint(bad_data)
     +                  .and.
     +     nint(masir4) .ne.  nint(bad_data)) then

        nmtests = nmtests + 1
        call set_qa_bit(qa_bits,19)
        mas11_4 = masir11 - masir4

c       Use polar night snow thresholds here.  Logic is that land
c       surfaces poleward of 60 latitude are snow covered most of
c       the year and many times ancillary snow map is incomplete.

        call get_pn_thresholds(masir11,locut,hicut,midpt,power)

        if (mas11_4 .le. midpt) then
          call set_bit(testbits,19)
          nptests = nptests + 1
        end if
        call conf_test(mas11_4,locut,hicut,power,midpt,1,c3)
        cmin2 = min(cmin2,c3)
        ngtests(2) = ngtests(2) + 1
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(1x,''mas11_4: '',6f10.2)')mas11_4,locut,
     +            hicut,midpt,power,masir11
      end if

c *******     END OF GROUP 2 TESTS ****************************



c *******    START OF GROUP 5 TESTS  **************************
c ... 4-12um brightness temperature difference test
c ... for thin cirrus)
      if (nint(masir12) .ne. nint(bad_data)
     +                  .and.
     +     nint(masir4) .ne.  nint(bad_data)) then
        mas4_12 = masir4 - masir12
        nmtests = nmtests + 1
        call set_qa_bit(qa_bits,17)
        if (mas4_12 .le. pnl4_12hi(2)) then
          nptests = nptests + 1
          call set_bit(testbits,17)
        end if
        call conf_test(mas4_12,pnl4_12hi(1),pnl4_12hi(3),pnl4_12hi(4),
     +                pnl4_12hi(2),1,c4)
        cmin5 = min(cmin5,c4)
        ngtests(3) = ngtests(3) + 1
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
         write(h_output,'(1x,''mas4_12: '',5f10.2)')mas4_12,pnl4_12hi(1),
     +            pnl4_12hi(2),pnl4_12hi(3),pnl4_12hi(4)
      endif
c ................................................................
c ********    END OF GROUP 5 TESTS  *****************************

c     Determine final confidence based on group values
      pre_confdnc = cmin1 * cmin2 * cmin5

c     Next, make sure you have all groups covered
      groups = 0
      do kk = 1,3
        if(ngtests(kk) .gt. 0) then
          groups = groups + 1.0
        end if
      enddo
      if (groups .gt. 0) fac = 1.0 / groups
c     Find final pixel confidence as nth root of group tests
      confdnc = pre_confdnc**fac


c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(1x,''tests '',5i10)') nmtests,nptests,ngtests
        write(h_output,'(1x,''confdnc '',8f8.5/,f8.5)') c2,c3,c4,c5,
     +         cmin1,cmin2,cmin5,fac,confdnc
      endif
c ................................................................

      return
      end
