C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      SUBROUTINE CLEAR_RADIANCE(
     &  LAT, LON, DAY_FLAG, NSTATS, NBANDS, NCELLS_MAX, PIXEL_DATA,
     &  NCELLS_DAY, NCELLS_NIGHT, CLEAR_DAY, CLEAR_NIGHT,
     &  INDEX_DAY, INDEX_NIGHT )

c-----------------------------------------------------------------------
c!F77
c
c!DESCRIPTION:
c
c!INPUT PARAMETERS:
c    LAT               Latitude (degrees, -90S to +90N)
c    LON               Longitude (degrees, -180W to 180E, Greenwich=0)
c    DAY_FLAG          Day/night flag
c                      .TRUE. => Day
c                      .FALSE. => Night
c    NSTATS            Number of statistics parameters to be computed
c                      (Dimension 1 of array PIXEL_DATA)
c                      (Currently FOUR statistics are computed)
c    NBANDS            Number of bands to be processed
c                      (Dimension 2 of array PIXEL_DATA)
c    NCELLS_MAX        Maximum allowable number of grid cells
c                      (Dimension 3 of array PIXEL_DATA)
c                      (For 5 min. of MODIS L1B 1km pixels, use 20000)
c    PIXEL_DATA        Array containing data for this pixel
c                      (selection of bands and units is left to caller)
c
c!OUTPUT PARAMETERS:
c    NCELLS_DAY        Number of day mode grid cells found so far
c                      (Updated continuously by this subroutine)
c    NCELLS_NIGHT      Number of night mode grid cells found so far
c                      (Updated continuously by this subroutine)
c    CLEAR_DAY         Statistics for day grid cells found so far
c                      Statistic 1: Number of pixels in this cell
c                      Statistic 2: Sum of pixel values in this cell
c                      Statistic 3: Minimum pixel value in this cell
c                      Statistic 2: Maximum pixel value in this cell
c                      (Updated continuously by this subroutine)
c    CLEAR_NIGHT       Statistics for night grid cells found so far
c                      Statistic 1: Number of pixels in this cell
c                      Statistic 2: Sum of pixel values in this cell
c                      Statistic 3: Minimum pixel value in this cell
c                      Statistic 2: Maximum pixel value in this cell
c                      (Updated continuously by this subroutine)
c    INDEX_DAY         Array of global grid index values for
c                      all day grid cells found so far
c                      (Updated continuously by this subroutine)
c    INDEX_NIGHT       Array of global grid index values for
c                      all night grid cells found so far
c                      (Updated continuously by this subroutine)
c
c!REVISION HISTORY:
c
c!TEAM-UNIQUE HEADER:
c    Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
c
c!DESIGN NOTES:
c    Original version by Liam.Gumley@ssec.wisc.edu
c
c!END
c-----------------------------------------------------------------------

      implicit none

c ... Common statement for debug purposes
      common / bug / debug, h_output

c ... Input parameters

      real lat, lon

      logical day_flag

      integer nstats, nbands, ncells_max

      real pixel_data( nbands )
      
c ... Output parameters

      integer ncells_day
      integer ncells_night
      
      real clear_day( nstats, nbands, ncells_max ),
     &     clear_night( nstats, nbands, ncells_max )

      integer index_day( ncells_max ),
     &        index_night( ncells_max )

c ... Local variables

      logical init

c ... (See get_grid_index.f for grid size definition)

      integer grid_size, h_output, debug
      parameter ( grid_size = 814880 )
      integer pointer_day( grid_size ), pointer_night( grid_size )

      integer i, j, k

      integer result, grid_index

      character*10 value_text
      
      integer band

      real stats( 20 )

c ... External functions

      integer get_grid_index
      external get_grid_index
            
c ... Save statement is required because
c ... this routine accumulates statistics

      save
            
c ... Data statements

      data init / .true. /
      
c-----------------------------------------------------------------------
c     INITIALIZATION      
c-----------------------------------------------------------------------

      if ( init ) then
       
c ...   Initializa grid index
        grid_index = 0

c ...   Initialize the output arrays

        do k = 1, ncells_max
          do j = 1, nbands
            do i = 1, nstats
              clear_day( i, j, k ) = 0.0
              clear_night( i, j, k ) = 0.0
            end do
          end do
        end do
 
        do i = 1, ncells_max
          index_day( i ) = 0
          index_night( i ) = 0
        end do
 
c ...   Initialize the local arrays

        do i = 1, grid_size
          pointer_day( i ) = 0
          pointer_night( i ) = 0
        end do

c ...   Set the number of day and night grid cells found

        ncells_day = 0
        ncells_night = 0

c ...   Turn off initialization flag

        init = .false.

      endif

c ... Given lat and lon, compute global grid index
      result = get_grid_index( lat, lon, grid_index )
      if ( result .ne. 0 ) then
        write( value_text, '( i10 )' ) result
        call message( 'clear_radiance',
     &    'Bad result code returned from get_grid_index (' //
     &    value_text // '); returning to caller [OPERATOR ACTION:' //
     &    'Contact SDST]', 0, 1 )
        return
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(5x,'' Clear Radiance:Day,GI '', L1,I10)')
     +        day_flag,grid_index
      endif
c ................................................................

      
c-----------------------------------------------------------------------
c     BEGIN PROCESSING
c-----------------------------------------------------------------------

c ... Decide whether to process in day mode or night mode

      if ( day_flag ) then

c---------------      
c ...   Day mode
c---------------      

c ...   Check whether this grid cell has been used yet

        if ( pointer_day( grid_index ) .eq. 0 ) then
          ncells_day = ncells_day + 1
          pointer_day( grid_index ) = ncells_day
        endif

c ...   Check if the maximum number of cells was exceeded

        if ( ncells_day .gt. ncells_max ) then
          write( value_text, '( i10 )' ) ncells_day
          call message( 'clear_radiance',
     &      'Maximum number of grid cells exceeded (' //
     &      value_text // ') [OPERATOR ACTION: Contact SDST]', 0, 2 )
        endif
        
c ...   If the grid index is not saved for this cell, save it

        if ( index_day( ncells_day ) .eq. 0 )
     &    index_day( ncells_day ) = grid_index

c ...   Loop over bands to compute statistics

        do band = 1, nbands
      
c ...     Get statistics for this cell

          do i = 1, nstats
            stats( i ) =
     &        clear_day( i, band, pointer_day( grid_index ) )
          end do

c ...     debug statement ............................................
          if (debug .gt. 0) then
              write(h_output,'(5x,'' stats: Day (B)  '',/,3i10,4f10.3)')
     +           i,band,pointer_day(grid_index),(stats(i),i=1,nstats) 
          endif
c ....................................................................

c ...     Compute statistics

          call clear_statistics( pixel_data( band ), nstats, stats )

c ...     Save statistics for this cell

          do i = 1, nstats
            clear_day( i, band, pointer_day( grid_index ) ) =
     &        stats( i )
          end do

c ...     debug statement ............................................
          if (debug .gt. 0) then
              write(h_output,'(5x,'' stats: Day (A) '',/,3i10,4f10.3)')
     +           i,band,pointer_day(grid_index),(stats(i),i=1,nstats) 
          endif
c ....................................................................

c ...   End of loop over bands
                                
        end do     

c ...   End of day mode
            
      else

c-----------------
c ...   Night mode
c-----------------

c ...   Check whether this grid cell has been used yet

        if ( pointer_night( grid_index ) .eq. 0 ) then
          ncells_night = ncells_night + 1
          pointer_night( grid_index ) = ncells_night
        endif

c ...   Check if the maximum number of cells was exceeded

        if ( ncells_night .gt. ncells_max ) then
          write( value_text, '( i10 )' ) ncells_night
          call message( 'clear_radiance',
     &      'Maximum number of grid cells exceeded (' //
     &      value_text // ') [OPERATOR ACTION: Contact SDST]', 0, 2 )
        endif
        
c ...   If the grid index is not saved for this cell, save it

        if ( index_night( ncells_night ) .eq. 0 )
     &    index_night( ncells_night ) = grid_index

c ...   Loop over bands to compute statistics

        do band = 1, nbands

c ...     Get statistics for this cell

          do i = 1, nstats
            stats( i ) =
     &        clear_night( i, band, pointer_night( grid_index ) )
          end do

c ...     debug statement ............................................
          if (debug .gt. 0) then
              write(h_output,'(5x,'' stats: Night(B) '',/,3i10,4f10.3)')
     +           i,band,pointer_night(grid_index),(stats(i),i=1,nstats) 
          endif
c ....................................................................

c ...     Compute statistics

          call clear_statistics( pixel_data( band ), nstats, stats )

c ...     Save statistics for this cell

          do i = 1, nstats
            clear_night( i, band, pointer_night( grid_index ) ) =
     &        stats( i )
          end do

c ...     debug statement ............................................
          if (debug .gt. 0) then
              write(h_output,'(5x,'' stats: Night(A) '',/,3i10,4f10.3)')
     +           i,band,pointer_night(grid_index),(stats(i),i=1,nstats) 
          endif
c ....................................................................

c ...   End of loop over bands
                                
        end do     

c ...   End of night mode
            
      endif
      
      END

c-----------------------------------------------------------------------

      SUBROUTINE CLEAR_STATISTICS( VALUE, NSTATS, STATS )

      implicit none

c-----------------------------------------------------------------------
c!F77
c
c!DESCRIPTION:   Calculated statistics for the current clear
c                 radiance grid point.
c
c!INPUT PARAMETERS:
c    VALUE             Current pixel clear radiance value
c    NSTATS            Number of statistics parameters to be computed
c                      (Currently FOUR statistics are computed)
c    STATS             Updated grid cell statistics
c                      Statistic 1: Number of pixels in this cell
c                      Statistic 2: Sum of pixel values in this cell
c                      Statistic 3: Minimum pixel value in this cell
c                      Statistic 2: Maximum pixel value in this cell
c
c!OUTPUT PARAMETERS:
c    STATS             Updated grid cell statistics
c                      Statistic 1: Number of pixels in this cell
c                      Statistic 2: Sum of pixel values in this cell
c                      Statistic 3: Minimum pixel value in this cell
c                      Statistic 2: Maximum pixel value in this cell
c
c!REVISION HISTORY:
c clear_stistics.f - 
c updated to add bad data checks 2/25/99 Kathy Strabala
c
c!TEAM-UNIQUE HEADER:
c    Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
c
c!DESIGN NOTES:
c    Original version by Liam.Gumley@ssec.wisc.edu
c
c!END
c-----------------------------------------------------------------------

      real value
      integer nstats
      real stats( nstats )

c ... Added check for missing or bad data

      if (value .gt. -99.0) then
c ...    Number of data values

         stats( 1 ) = stats( 1 ) + 1

c ...    Sum of data values

         stats( 2 ) = stats( 2 ) + value

c ...    Minimum of data values

         if (nint(stats(1)) .eq. 1) then
           stats( 3 ) = min( 1000.0, value )
         else
           stats( 3 ) = min( stats( 3 ), value )
         endif

c ...    Maximum of data values

         stats( 4 ) = max( stats( 4 ), value )

      endif

      END
