C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center,
C  University C  of Wisconsin-Madison, Madison WI.
C
C  This program is free software; you can redistribute it
C  and/or modify it under the terms of the GNU General
C  Public License as published by the Free Software Foundation;
C  either version 2 of the License, or (at your option) any
C  later version.
C
C  This program is distributed in the hope that it will be
C  useful, but WITHOUT ANY WARRANTY; without even the implied
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public
C  License along with this program; if not, write to the Free
C  Software Foundation, Inc., 59 Temple Place, Suite 330,
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine snow_mask(pxldat,plat,plon,land,snglnt,water,hi_elev,
     +                     ndsi_snow)

      implicit none
      save

c ... Common statement for debug purposes
      common / bug / debug, h_output

c---------------------------------------------------------------------
c!F77
c
c!Description:
c       Subroutine which implements a quick and dirty version of
c       the snow algorithm by Dorothy Hall, George Riggs and 
c       Vince Salmonson.
c
c!Input Parameters:
c pxldat        Array containing reflectance or brightness temperatures
c               for all bands for a single pixel
c plat          Latitude of current pixel
c plon          Longitude of current pixel
c land          Flag indicating a pixel to be processed as land
c snglnt        Logical variable flagging background as sunglint 
c               contaminated
c water         Logical variable indicating water surface for current 
c               pixel
c hi_elev       Logical flag indicating elevation > 2000 meters
c
c!Output Parameters:
c ndsi-snow     logical variable indicating the prescence of snow in a
c               FOV according to the NDSI test
c
c!Revision History:
c
c!Team-unique Header:
c
c!References and Credits:
c See snow product ATBD.
c
c!END
c---------------------------------------------------------------------

      include 'global.inc'
      include 'snow_mask.inc'
      include 'platform_name.inc'
c
c ... scalar arguments ..
      logical snglnt,water,ndsi_snow,hi_elev,land
      real plat,plon
c ...
c ... array arguments ..
      real pxldat(inband)
c ...
c ... local scalars ..
      real ndsi,masv88,masv55,masv188,masir13,masir11,
     +     masir85,masir37,diff,sth37_11,masnir
      integer h_output,debug,I_bad
      logical Greenland
c
c ... identification 
      masv55 = pxldat(4)
      masv88 = pxldat(2) 
      masv188 = pxldat(26)
      masir37 = pxldat(20)
      masir85 = pxldat(29)
      masir11 = pxldat(31)
      masir13 = pxldat(35)
c     Select NIR channel for NDSI based on platform name.
c     Band 6 for Terra, band 7 for Aqua.
      if(platform_name .eq. 'Terra') then
        masnir = pxldat(6)
      else if(platform_name .eq. 'Aqua ') then
        masnir = pxldat(7)
      else
        call message( 'snow_mask', 'Platform name not recognized' //
     &    ' [OPERATOR ACTION: Contact SDST]', 0, 2)
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(10x/,''Subroutine Snow_Mask '',/)')
      endif
c ................................................................

c
c ... Initialize the ndsi value and snow mask result.
      ndsi = 0.0
      ndsi_snow = .false.
      I_bad = nint(bad_data)

c ... First, check the 11 micron brightness temperature.
      if (nint(masir11) .ne. I_bad .and. masir11 .le. sm_bt11(1)) then

c ...    perform the NDSI on the current pixel
         if (nint(masv55) .ne. I_bad .and. nint(masnir).ne.I_bad) then
           ndsi  = (masv55 - masnir) / (masv55 + masnir)

           if ( (ndsi .gt. sm_ndsi(1)) .and. (masv88 .gt. sm_ref2(1)) ) 
     +     then

             ndsi_snow = .true.

c            Check for false snow detection.

c            Define "Greenland".
             Greenland = .false.
             if(land) then
               if(plat .ge. 60.0 .and. plat .lt. 67.0) then
                 if(plon .ge. -60.0 .and. plon .lt. -30.0) then
                   Greenland = .true.
                 end if
               else if(plat .ge. 67.0 .and. plat .lt. 75.0) then
                 if(plon .ge. -60.0 .and. plon .lt. -10.0) then
                   Greenland = .true.
                 end if
               else if(plat .ge. 75.0) then
                 if(plon .ge. -70.0 .and. plon .lt. -10.0) then
                   Greenland = .true.
                 end if
               end if
             end if

             if( (.not. hi_elev) .and. (.not. Greenland) ) then
               if(masnir .gt. sm_mnir(1)) then
                 ndsi_snow = .false.
               end if
             end if

c            Now, make sure NDSI is not flagging a thin cirrus
             if(nint(masv188).ne.I_bad.and.nint(masir13).ne.I_bad) then
               if (masv188 .gt. sm_ref3(1) .and. masir13 .lt. sm_co2(1)) then
                  ndsi_snow = .false.
               endif
             endif
c            If in sunglint region, disregard if between -60 and 50 lat.
             if(water .and. snglnt .and. plat .le. 50.0 .and.
     +          plat .ge. -60.0) then
               ndsi_snow = .false.
             end if
c            Check for ice clouds mis-identified as snow.
             if(masir85 .ne. I_bad .and. masir11 .ne. I_bad) then
               diff = masir85 - masir11
               if(diff .ge. sm85_11(1)) then
                 ndsi_snow = .false.
               end if
             end if
c            Check for water clouds mis-identified as snow.
             if(masir37 .ne. I_bad .and. masir11 .ne. I_bad) then
               diff = masir37 - masir11
               if(hi_elev) then
                 sth37_11 = sm37_11hel(1)
               else
                 sth37_11 = sm37_11(1)
               end if
               if(diff .ge. sth37_11) then
                 ndsi_snow = .false.
               end if
             end if

           endif 
         endif
      endif

c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(15x,'' snow mask variables '',L4)') ndsi_snow
        write(h_output,'(2x,''masv55 masnir ndsi masv188 masv88 masir13
     +      snglnt hi_elev ndsi_snow '',/,5f12.4,f7.2,3L8)') masv55,masnir,ndsi,
     +                   masv188,masv88,masir13,snglnt,hi_elev,ndsi_snow
        write(h_output,'(''IR vals '',3f10.2)') masir37,masir11,masir85
      endif
c ................................................................

      return
      end
